defmodule DecentralisedBookIndex.Sync.PublisherSync do
  alias DecentralisedBookIndex.Metadata
  alias DecentralisedBookIndex.Metadata.Publisher

  def create_update(attrs, server_id) do
    case Metadata.get_publisher_by_id(attrs.id) do
      {:ok, publisher} ->
        if not is_local(publisher) and is_new(attrs, publisher) do
          attrs =
            attrs
            |> Map.delete(:id)
            |> Map.put_new(:dbi_server_id, server_id)

          publisher
          |> Ash.Changeset.for_update(:sync, attrs)
          |> Ash.update!(authorize?: false)
        end

        :ok

      {:error, %Ash.Error.Query.NotFound{}} ->
        attrs =
          attrs
          |> Map.put_new(:dbi_server_id, server_id)

        Publisher
        |> Ash.Changeset.for_create(:sync_create, attrs)
        |> Ash.create!(authorize?: false)

        :ok
    end
  end

  defp is_local(publisher) do
    is_nil(publisher.dbi_server_id)
  end

  defp is_new(new_publisher, publisher) do
    with {:ok, datetime, _offset} <- DateTime.from_iso8601(new_publisher[:updated_at]) do
      DateTime.after?(datetime, publisher.updated_at)
    else
      {:error, reason} ->
        false
    end
  end
end
