defmodule DecentralisedBookIndexWeb.Components.MyComponents.SelectBookEdition do
  use DecentralisedBookIndexWeb, :live_component
  alias DecentralisedBookIndex.Metadata

  @impl true
  def render(assigns) do
    ~H"""
    <div>
      <.header>
        Select another edition of the book
      </.header>

      <form class="flex items-center" phx-change="search" phx-target={@myself}>
        <label for="simple-search" class="sr-only">Search</label>
        <div class="relative w-full">
          <div class="absolute inset-y-0 left-0 flex items-center pl-3 pointer-events-none">
            <svg
              aria-hidden="true"
              class="w-5 h-5 text-gray-500 dark:text-gray-400"
              fill="currentColor"
              viewBox="0 0 20 20"
              xmlns="http://www.w3.org/2000/svg"
            >
              <path
                fill-rule="evenodd"
                d="M8 4a4 4 0 100 8 4 4 0 000-8zM2 8a6 6 0 1110.89 3.476l4.817 4.817a1 1 0 01-1.414 1.414l-4.816-4.816A6 6 0 012 8z"
                clip-rule="evenodd"
              >
              </path>
            </svg>
          </div>
          <input
            name="query"
            type="text"
            phx-debounce="300"
            autocomplete="off"
            id="simple-search"
            class="block w-full p-2 pl-10 text-sm text-gray-900 border border-gray-300 rounded-lg bg-gray-50 focus:ring-primary-500 focus:border-primary-500 dark:bg-gray-700 dark:border-gray-600 dark:placeholder-gray-400 dark:text-white dark:focus:ring-primary-500 dark:focus:border-primary-500"
            placeholder="Search"
            value=""
          />
        </div>
      </form>

      <%= if Enum.empty?(@page.results) do %>
        <div class="flex justify-center ">
          <p class="text-lg font-semibold py-5 dark:text-white">
            No Books
          </p>
        </div>
      <% else %>
        <%= for book_edition <- @page.results do %>
          <div
            phx-click="select-book-edition"
            phx-target={@notify_component}
            phx-value-book-edition={book_edition.book_editions_registry_id}
            class="w-full bg-white border border-gray-200 rounded-lg shadow-sm dark:bg-gray-800 dark:border-gray-700 mx-auto my-3 hover:bg-gray-100 dark:hover:bg-gray-700"
          >
            <h5 class="mb-1 text-lg font-medium text-gray-900 dark:text-white px-2 pt-1">
              {book_edition.title}
            </h5>
          </div>
        <% end %>
      <% end %>
    </div>
    """
  end

  @impl true
  def update(assigns, socket) do
    {:ok,
     socket
     |> assign(assigns)
     |> assign(:book_edition_query, "")
     |> search()}
  end

  def handle_event("search", %{"query" => query}, socket) do
    {:noreply,
     socket
     |> assign(:book_edition_query, query)
     |> search()}
  end

  defp search(socket) do
    query = Map.get(socket.assigns, :book_edition_query, "")
    page = Metadata.search_book!(query)

    socket
    |> assign(page: page)
  end
end
