defmodule DecentralisedBookIndexWeb.BookLive.Edit do
  use DecentralisedBookIndexWeb, :live_view

  @impl true
  def render(assigns) do
    ~H"""
    <.live_component
      module={DecentralisedBookIndexWeb.BookLive.FormComponent}
      id={(@book && @book.id) || :new}
      title={@page_title}
      current_user={@current_user}
      action={@live_action}
      book={@book}
      alternative_book_id={@alternative_book_id}
    />
    """
  end

  @impl true
  def mount(params, _session, socket) do
    alternative_book_id = Map.get(params, "alternative_book_id", nil)

    {:ok,
     socket
     |> assign(:alternative_book_id, alternative_book_id)
     |> assign_new(:current_user, fn -> nil end)}
  end

  @impl true
  def handle_params(params, _url, socket) do
    socket =
      socket
      |> apply_action(socket.assigns.live_action, params)

    {:noreply, socket}
  end

  defp apply_action(socket, :edit, %{"id" => id}) do
    socket
    |> assign(:page_title, "Edit Book")
    |> assign(
      :book,
      Ash.get!(DecentralisedBookIndex.Metadata.Book, id, load: [:dbi_server], actor: socket.assigns.current_user)
    )
    |> redirect_if_not_editable()
  end

  defp apply_action(socket, :new, _params) do
    socket
    |> assign(:page_title, "New Book")
    |> assign(:book, nil)
  end

  @impl true
  def handle_info({DecentralisedBookIndexWeb.BookLive.FormComponent, {:saved, _book}}, socket) do
    {:noreply, socket}
  end

  defp redirect_if_not_editable(socket) do
    if not is_nil(socket.assigns.book.dbi_server) do
      socket =
        socket
        |> Phoenix.LiveView.put_flash(:error, "Can't edit other server's data!")
        |> Phoenix.LiveView.redirect(to: ~p"/books/#{socket.assigns.book}")
    else
      socket
    end
  end
end
