defmodule DecentralisedBookIndex.Sync.ApiClients.FetchJsonsTest do
  use DecentralisedBookIndex.DataCase

  alias DecentralisedBookIndex.Sync.ApiClients.FetchJsons
  alias DecentralisedBookIndex.Metadata

  alias DecentralisedBookIndex.TestEndpoints

  @test_server_endpoint TestEndpoints.test_api_endpoint()

  setup do
    user = generate(user(role: :moderator))
    %{user: user}
  end

  describe "authors api" do
    test "get authors", %{user: user} do
      for number <- 1..11 do
        Metadata.create_author!("Author#{number}", "An description#{number}", nil, nil,
          actor: user
        )
      end

      assert {:ok, records} = FetchJsons.get("#{@test_server_endpoint}/api/v1/json/authors")
      assert length(records) == 11
    end

    test "get authors names from API", %{user: user} do
      for number <- 1..11 do
        Metadata.create_author!("Author#{number}", "An description#{number}", nil, nil,
          actor: user
        )
      end

      get_author_names = fn data ->
        Enum.map(data, fn author -> author["attributes"]["name"] end)
      end

      assert {:ok, records} =
               FetchJsons.get("#{@test_server_endpoint}/api/v1/json/authors", get_author_names)

      assert length(records) == 11
      assert Enum.all?(records, fn author -> String.contains?(author, "Author") end)
    end
  end
end
