defmodule DecentralisedBookIndex.Sync.DataTransformers.AuthorRoleTransformerTest do
  use ExUnit.Case, async: true

  alias DecentralisedBookIndex.Sync.DataTransformers.AuthorRoleTransformer

  describe "correct transformations" do
    test "a json contains correct author role information" do
      json_body =
        %{
          "data" => %{
            "attributes" => %{
              "author_id" => "d6d784a9-4c9c-45a3-8068-a5a30bbe8934",
              "order" => 0,
              "role" => ""
            },
            "id" => "eafd6569-aaac-442d-a64a-8df17051f0bc",
            "links" => %{},
            "meta" => %{},
            "relationships" => %{"author" => %{"links" => %{}, "meta" => %{}}},
            "type" => "author_role"
          }
        }

      assert {:ok, author_role} = AuthorRoleTransformer.from_json(json_body)

      assert %{
               id: "eafd6569-aaac-442d-a64a-8df17051f0bc",
               order: 0,
               role: "",
               author_id: "d6d784a9-4c9c-45a3-8068-a5a30bbe8934"
             } = author_role
    end

    test "a json doesn't contains author role information \"data\" attribute" do
      json_body =
        %{
          "attributes" => %{
            "author_id" => "d6d784a9-4c9c-45a3-8068-a5a30bbe8934",
            "order" => 0,
            "role" => ""
          },
          "id" => "eafd6569-aaac-442d-a64a-8df17051f0bc",
          "links" => %{},
          "meta" => %{},
          "relationships" => %{"author" => %{"links" => %{}, "meta" => %{}}},
          "type" => "author_role"
        }

      assert {:ok, author_role} = AuthorRoleTransformer.from_json(json_body)

      assert %{
               id: "eafd6569-aaac-442d-a64a-8df17051f0bc",
               order: 0,
               role: "",
               author_id: "d6d784a9-4c9c-45a3-8068-a5a30bbe8934"
             } = author_role
    end
  end
end
